package hwdb;
#---------------------------------------------------------------
# Project         : Mandriva Linux
# Module          : hwdb/clients
# File            : hwdb.pm
# Version         : $Id: hwdb.pm,v 1.39 2005/09/13 09:40:30 flepied Exp $
# Author          : Frederic Lepied
# Created On      : Wed Jan  7 20:48:18 2004
# Purpose         : common functions
#---------------------------------------------------------------

# be sure to have the messages in english to avoid having unknown printer
# translated in the all languages inside the database.
$ENV{LANGUAGE} = 'C';

use SOAP::Lite;
use POSIX;
use Term::ReadKey;

use lib qw(/usr/lib/libDrakX);
use detect_devices;
use standalone;

use MDK::Common;
use printer::detect;

my $user = $ARGV[0];
my $pass;
my @components;

$ENV{HTTP_proxy} = $ENV{http_proxy};

sub int16 {
    my ($n) = @_;
    return sprintf("%04x", $n);
}

sub SOAP::Transport::HTTP::Client::get_basic_credentials() { 
    return $user => $pass;
}

# detection routines

sub detect {
    my ($type) = @_;
    
    require harddrake::data;

    push @harddrake::data::tree, {
	class => "MONITOR",
	detector => sub {
	    require Xconfig::monitor;
	    if (defined(&Xconfig::monitor::probe)) {
		Xconfig::monitor::probe();
	    } else {
		Xconfig::monitor::getinfoFromDDC();
	    }
	},
    };

    foreach (@harddrake::data::tree) {
	my $Ident = $_->{class};
	my $detector = $_->{detector};
	next if $type && $type ne $Ident;

        # No detector ? (should never happen but who knows ?)
        ref($detector) eq 'CODE' or next;
	
        my @devices;

	# Detect only local devices
 	if ($Ident eq 'PRINTER') {
 	    @devices = printer::detect::local_detect();
 	} else {
	    @devices = &$detector;
 	}

	# copy detected structure to allow modification
	return map { my %device = %$_; \%device }  @devices if $type;

        @devices or next;

	use Data::Dumper;
	print 'Probing ', $Ident, " class\n";
	print Dumper(\@devices);
    }
}

sub upload_video_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('VIDEO')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	print "Uploading video card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	$r = $hwdb->add_video_card($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
	print "done [", $r->result, "]\n";
	push(@components, $r->result);
    }
}

sub upload_network_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('ETHERNET')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	if ($card->{bus} eq 'PCI') {
	    $card->{vendor} = int16($card->{vendor});
	    $card->{id} = int16($card->{id});
	    $card->{subvendor} = int16($card->{subvendor});
	    $card->{subid} = int16($card->{subid});
	    print "Uploading PCI network card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	    $r = $hwdb->add_pci_network_card($card->{description}, $card->{vendor}, $card->{id}, $card->{subvendor}, $card->{subid});
	} elsif ($card->{bus} eq 'USB') {
	    $card->{vendor} = int16($card->{vendor});
	    $card->{id} = int16($card->{id});
	    print "Uploading USB network card $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_network_card($card->{description}, $card->{vendor}, $card->{id});
	} else {
	    print "unknown bus (ethernet): $card->{bus} $card->{description} ($card->{vendor}:$card->{id})\n";
	}
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_sound_card {
    my ($hwdb) = @_;
    
    foreach my $card (detect('AUDIO')) {
	my $r;
	$card->{description} =~ s/.*\|//;
	if ($card->{bus} eq 'PCI') {
	    print "Uploading PCI audio card $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	    $r = $hwdb->add_pci_sound_card($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
	} elsif ($card->{bus} eq 'USB') {
	    print "Uploading USB audio card $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_sound_card($card->{description}, int16($card->{vendor}), int16($card->{id}));
	} else {
	    print "unknown bus (audio): $card->{bus} $card->{description} ($card->{vendor}:$card->{id})\n";
	}
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_scanner {
    my ($hwdb) = @_;

    foreach my $info (detect('SCANNER')) {
	my $r;
	my $card = $info->{val};
	
	if ($info->{port} =~ /usb/) {
	    $card->{DESCRIPTION} =~ s/.*\|//;
	    $card->{vendor} =~ s/0x//;
	    $card->{id} =~ s/0x//;
	    print "Uploading USB scanner $card->{DESCRIPTION} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_scanner($card->{DESCRIPTION}, $card->{vendor}, $card->{id});
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_reader {
    my ($hwdb, $type, $func) = @_;

    foreach my $info (detect($type)) {
	my $r;

	if ($info->{info}) {
	    if (!$info->{Vendor}) {
		my @a = split(' ', $info->{info});
		$info->{Vendor} = ucfirst(lc($a[0]));
		shift @a;
		$info->{Model} = join(' ', @a);
	    }
	    if (!$info->{Model}) {
		$info->{Model} = $info->{info};
	    }
	    print "Uploading $type $info->{Model} ($info->{Vendor}:$info->{bus})...";
	    $r = $func->($info->{Model}, $info->{Vendor}, $info->{info}, $info->{bus});
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_cdrom {
    my ($hwdb) = @_;

    upload_reader($hwdb, 'CDROM', sub { $hwdb->add_cdrom(@_) });
}

sub upload_dvdrom {
    my ($hwdb) = @_;

    upload_reader($hwdb, 'DVDROM', sub { $hwdb->add_dvdrom(@_) });
}

sub upload_burner {
    my ($hwdb) = @_;

    upload_reader($hwdb, 'BURNER', sub { $hwdb->add_burner(@_) });
}

sub upload_printer {
    my ($hwdb) = @_;

    foreach my $info (detect('PRINTER')) {
	my $r;
	my $card = $info->{val};

	$card->{MODEL} =~ s/.*\|//;
	
	if ($card->{MANUFACTURER} ne '') {
	    if ($info->{port} =~ /usb/) {
		print "Uploading USB printer $card->{MODEL}...";
		$r = $hwdb->add_usb_printer2($card->{MODEL}, $card->{MANUFACTURER}, $card->{IEEE1284});
	    } elsif ($info->{port} =~ m!/dev/lp!) {
		print "Uploading parallel printer $card->{MODEL}...";
		$r = $hwdb->add_parallel_printer2($card->{MODEL}, $card->{MANUFACTURER}, $card->{IEEE1284});
	    } else {
		print "unknown printer port: $info->{port}\n";
	    }
	} else {
	    use Data::Dumper;
	    print "Unsupported printer please report:\n";
	    print Dumper(\$card);
	}
	
	if ($r) {
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_webcam {
    my ($hwdb) = @_;

    foreach my $card (detect('WEBCAM')) {
	my $r;
	
	$card->{description} =~ s/.*\|//;
	print "Uploading USB webcam $card->{description} ($card->{vendor}:$card->{id})...";
	$r = $hwdb->add_webcam($card->{description}, int16($card->{vendor}), int16($card->{id}));
        print "done [", $r->result, "]\n";
        push(@components, $r->result);
    }
}

sub upload_dmi {
    my ($hwdb) = @_;
    my $info;
    my $r;
    
    foreach (`dmidecode`) {
	chomp($_);
	
	# BIOS info
	if (/^\tDMI type 0,/ ... /^Handle/) {
	    $info->{bios} = $2 if /.*(vendor|manufacturer):\s*(\S.+?)\s*$/i;
	    $info->{biosver} = $1 if /.*Version:\s*(\S.+?)\s*$/i;
	    $info->{date} = $1 if /.*Date:\s*(\S.+?)\s*$/i;
	    if (/^Handle/ && $info->{bios} && $info->{biosver}) {
		print "Uploading BIOS $info->{bios} ($info->{biosver}:$info->{date})...";
		$r = $hwdb->add_bios($info->{bios}, $info->{biosver}, $info->{date});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{bios} = 0;
	    }
	}
	
	# system info
	if (/^\tDMI type 1,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if /.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i;
	    $info->{version} = $1 if /.*Version:\s*(\S.+?)\s*$/i;
	    $info->{product} = $1 if /.*Product.*:\s*(\S.+?)\s*$/i;
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading system $info->{manufacturer}:$info->{product}:$info->{version}...";
		if ($info->{version}) {
		    $r = $hwdb->add_system_info($info->{manufacturer}, $info->{version}, $info->{product});
		} else {
		    $r = $hwdb->add_system_info($info->{manufacturer}, $info->{product}, $info->{version});
		}
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
		$info->{version} = 0;
	    }
	}

	# board info
	if (/^\tDMI type 2,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if /.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i;
	    $info->{product} = $1 if /.*Product.*:\s*(\S.+?)\s*$/i;
	    $info->{version} = $1 if /.*Version:\s*(\S.+?)\s*$/i;
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading board $info->{manufacturer}:$info->{product}:$info->{version}...";
		$r = $hwdb->add_board($info->{manufacturer}, $info->{product}, $info->{version});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
	    }
	}

	# Chassis info
	if (/^\tDMI type 3,/ ... /^Handle/) {
	    $info->{manufacturer} = $2 if /.*(vendor|Manufacturer):\s*(\S.+?)\s*$/i;
	    $info->{type} = $1 if /.*Type*:\s*(\S.+?)\s*$/i;
	    if (/^Handle/ && $info->{manufacturer}) {
		print "Uploading chassis $info->{manufacturer}:$info->{type}...";
		$r = $hwdb->add_chassis($info->{manufacturer}, $info->{type});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
		$info->{manufacturer} = 0;
	    }
	}
    }
}

sub upload_usb_storage {
    my ($hwdb) = @_;

    foreach my $card (detect('HARDDISK')) {
	my $r;

	if ($card->{usb_description} && $card->{usb_vendor} && $card->{usb_id}) {
	    $card->{usb_description} =~ s/.*\|//;
	    $card->{usb_description} =~ s/\s*$//;
	    $card->{usb_vendor} = int16($card->{usb_vendor});
	    $card->{usb_id} = int16($card->{usb_id});
	    print "Uploading USB storage $card->{usb_description} ($card->{usb_vendor}:$card->{usb_id})...";
	    $r = $hwdb->add_usb_storage($card->{usb_description}, $card->{usb_vendor}, $card->{usb_id});
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_adsl {
    my ($hwdb) = @_;

    foreach my $card (detect('ADSL')) {
	my $r;

	if ($card->{bus} eq 'PCI') {
	    if ($card->{description} && $card->{vendor} && $card->{id} && $card->{subvendor} && $card->{subid}) {
		$card->{description} =~ s/.*\|//;
		$card->{description} =~ s/\s*$//;
		$card->{vendor} = int16($card->{vendor});
		$card->{id} = int16($card->{id});
		$card->{subvendor} = int16($card->{subvendor});
		$card->{subid} = int16($card->{subid});
		print "Uploading PCI ADSL device $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
		$r = $hwdb->add_pci_adsl($card->{description}, $card->{vendor}, $card->{id}, $card->{subvendor}, $card->{subid});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
	    }
	} else {
	    if ($card->{description} && $card->{vendor} && $card->{id}) {
		$card->{description} =~ s/.*\|//;
		$card->{description} =~ s/\s*$//;
		$card->{vendor} = int16($card->{vendor});
		$card->{id} = int16($card->{id});
		print "Uploading USB ADSL device $card->{description} ($card->{vendor}:$card->{id})...";
		$r = $hwdb->add_usb_adsl($card->{description}, $card->{vendor}, $card->{id});
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
	    }
	}
    }
}

sub upload_controllers {
    my ($hwdb) = @_;

    foreach my $type ('AGP', 'BRIDGE', 'ATA_STORAGE', 'SATA_STORAGE', 'FIREWIRE_CONTROLLER', 'USB_CONTROLLER', 'SCSI_CONTROLLER', 'UNKNOWN', 'RAID_STORAGE') {
	foreach my $card (detect($type)) {
	    my $r;
	    
	    $card->{description} =~ s/.*\|//;
	    $card->{media_type} =~ s/SERIAL_USB/usb/ if $type eq 'USB_CONTROLLER';
	    $card->{media_type} =~ s/BRIDGE_HOST/agp/ if $type eq 'AGP';
	    $card->{media_type} =~ s/BRIDGE_PCI/pci/ if $type eq 'BRIDGE';
	    $card->{media_type} =~ s/BRIDGE_ISA/isa/ if $type eq 'BRIDGE';
	    $card->{media_type} =~ s/BRIDGE_CARDBUS/cardbus/ if $type eq 'BRIDGE';
	    if ($type eq 'ATA_STORAGE') {
		$card->{media_type} =~ s/STORAGE_IDE/ide/;
		$card->{media_type} =~ s/STORAGE_RAID/ideraid/;
	    }
	    if ($type eq 'SCSI_CONTROLLER' || $type eq 'UNKNOWN' || $type eq 'RAID_STORAGE') {
		$card->{media_type} =~ s/STORAGE_SCSI/scsi/;
		$card->{media_type} =~ s/STORAGE_RAID/scsiraid/;
	    }
	    $card->{media_type} =~ s/SERIAL_FIREWIRE/firewire/ if $type eq 'FIREWIRE_CONTROLLER';
	    $card->{media_type} =~ s/STORAGE_RAID/sataraid/ if $type eq 'SATA_STORAGE';
	    
	    if ($card->{bus} eq 'PCI' && $card->{media_type} =~ /^[a-z]+$/) {
		my $method = "add_$card->{media_type}_controller";

		print "Uploading $card->{media_type} controller $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
		$r = $hwdb->$method($card->{description}, int16($card->{vendor}), int16($card->{id}), int16($card->{subvendor}), int16($card->{subid}));
		print "done [", $r->result, "]\n";
		push(@components, $r->result);
	    } else {
		use Data::Dumper;
		print "Unsupported controller please report:\n";
		print Dumper(\$card);
	    }
	}
    }
}

sub upload_cpu {
    my ($hwdb) = @_;

    foreach my $card (detect('CPU')) {
	my $r;
	
	print "Uploading CPU $card->{'model name'} ($card->{vendor_id}, $card->{'cpu MHz'}, $card->{'cpu family'},  $card->{model},  $card->{stepping})...";
	$r = $hwdb->add_cpu($card->{'model name'}, $card->{vendor_id}, $card->{'cpu MHz'}, $card->{'cpu family'},  $card->{model},  $card->{stepping});
	print "done [", $r->result, "]\n";
	push(@components, $r->result);
    }
}

sub upload_modem {
    my ($hwdb) = @_;

    foreach my $card (detect('MODEM')) {
	my $r;
	
	if ($card->{bus} eq 'PCI') {
	    $card->{description} =~ s/.*\|//;
	    $card->{vendor} = int16($card->{vendor});
	    $card->{id} = int16($card->{id});
	    $card->{subvendor} = int16($card->{subvendor});
	    $card->{subid} = int16($card->{subid});
	    print "Uploading PCI modem $card->{description} ($card->{vendor}:$card->{id} $card->{subvendor}:$card->{subid})...";
	    $r = $hwdb->add_pci_modem($card->{description}, $card->{vendor}, $card->{id}, $card->{subvendor}, $card->{subid});
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_ups {
    my ($hwdb) = @_;

    foreach my $card (detect('UPS')) {
	my $r;
	
	if ($card->{bus} eq 'USB') {
	    $card->{description} =~ s/.*\|//;
	    print "Uploading USB UPS $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_ups($card->{description}, int16($card->{vendor}), int16($card->{id}));
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
	# TODO add_serial_ups()
    }
}

sub upload_mouse {
    my ($hwdb) = @_;

    foreach my $card (detect('MOUSE')) {
	my $r;
	
	if ($card->{bus} eq 'USB') {
	    $card->{description} =~ s/.*\|//;
	    if ($card->{description} =~ /tablet/i || $card->{driver} =~ /tablet/i ) {
		print "Uploading USB tablet $card->{description} ($card->{vendor}:$card->{id})...";
		$r = $hwdb->add_usb_tablet($card->{description}, int16($card->{vendor}), int16($card->{id}));
	    } else {
		print "Uploading USB mouse $card->{description} ($card->{vendor}:$card->{id})...";
		$r = $hwdb->add_usb_mouse($card->{description}, int16($card->{vendor}), int16($card->{id}));
	    }
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_keyboard {
    my ($hwdb) = @_;

    foreach my $card (detect('KEYBOARD')) {
	my $r;
	
	if ($card->{bus} eq 'USB') {
	    $card->{description} =~ s/.*\|//;
	    print "Uploading USB keyboard $card->{description} ($card->{vendor}:$card->{id})...";
	    $r = $hwdb->add_usb_keyboard($card->{description}, int16($card->{vendor}), int16($card->{id}));
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_pcmcia_card {
    my ($hwdb, $info) = @_;

    my @tab = ("multifunction", "memory", "modem", "parallel",
	       "harddisk", "video", "network", "aims", "scsi");

    if ($info->{vendor} && $info->{model} && $info->{rev} && $info->{id} && $info->{manfid1} && $info->{manfid2} && $info->{function}) {
	if ($info->{function} < 9) {
	    print ($info->{vendor} , $info->{model} , $info->{rev} , $info->{id} , $info->{manfid1} , $info->{manfid2} , $tab[$info->{function}]);
	}
    }
}

sub upload_pcmcia {
    my ($hwdb) = @_;
    my $info;
    my $r;

    if ( -x '/sbin/cardctl' ) {
	foreach (`/sbin/cardctl ident`) {
	    chomp($_);
	    
	    if (/^Socket .:/) {
		upload_pcmcia_card($hwdb, $info);
		$info = 0;
	    }
	    ($info->{vendor}, $info->{model}, $info->{rev}, $info->{id}) = ($1, $2, $3, $4) if /product info: "(.*?)", "(.*?)", "(.*?)", "(.*?)"/;
	    ($info->{manfid1}, $info->{manfid2}) = ($1, $2) if /manfid: (.*), (.*)/;
	    $info->{function} = $1 if /function: ([0-9]*)/;
	}
	upload_pcmcia_card($hwdb, $info);
    }
}

sub upload_unknown {
    my ($hwdb) = @_;

    foreach my $card (detect('UNKNOWN')) {
	my $r;
	
	if ($card->{bus} eq 'PCI') {

	    $card->{description} =~ s/.*\|//;
	    my $vendor = int16($card->{vendor});
	    my $id = int16($card->{id});
	    my $subvendor = int16($card->{subvendor});
	    my $subid = int16($card->{subid});

	    if ($card->{media_type} eq 'COMMUNICATION_MODEM') {
		print "Uploading PCI modem $card->{description} ($vendor:$id $subvendor:$subid)...";
		$r = $hwdb->add_pci_modem($card->{description}, $vendor, $id, $subvendor, $subid);
	    } else {
		print "Uploading unknown PCI device $card->{description} ($vendor:$id $subvendor:$subid)...";
		$r = $hwdb->add_unknown_pci($card->{description}, $vendor, $id, $subvendor, $subid);
	    }
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}

	if ($card->{bus} eq 'USB') {
	    $card->{description} =~ s/.*\|//;
	    my $vendor = int16($card->{vendor});
	    my $id = int16($card->{id});
	    if ($card->{media_type} =~ /Mass Storage/) {
		print "Uploading USB storage $card->{usb_description} ($vendor:$id)...";
		$r = $hwdb->add_usb_storage($card->{usb_description}, $vendor, $id);
	    } else {
		print "Uploading unknown USB device $card->{description} ($card->{vendor}:$card->{id})...";
		$r = $hwdb->add_unknown_usb($card->{description}, $vendor, $id);
	    }
	    print "done [", $r->result, "]\n";
	    push(@components, $r->result);
	}
    }
}

sub upload_system {
    my ($hwdb) = @_;
    my $name = $ARGV[1];

    upload_cpu($hwdb);
    upload_dmi($hwdb);
    upload_video_card($hwdb);
    upload_network_card($hwdb);
    upload_sound_card($hwdb);
    upload_cdrom($hwdb);
    upload_dvdrom($hwdb);
    upload_burner($hwdb);
    upload_webcam($hwdb);
    upload_usb_storage($hwdb);
    upload_adsl($hwdb);
    upload_printer($hwdb);
    upload_controllers($hwdb);
    upload_modem($hwdb);
    upload_ups($hwdb);
    upload_mouse($hwdb);
    upload_keyboard($hwdb);
    # scanner and unknown must be called at the end because
    # they use information provided by other checks
    upload_scanner($hwdb);
    upload_unknown($hwdb);

    print "Uploading complete system $name...";
    $r = $hwdb->add_system($name, ' ' . join(' ', sort { $a <=> $b } @components) . ' ');
    print "done [", $r->result, "]\n";
}

# common init phase

sub init {
    my ($name, $count, $opt) = @_;

    !$count and $count = 1;
    
    if ($#ARGV != $count - 1) {
	print "usage: $name <account>$opt\n";
	exit 1;
    }

    my $hwdb = SOAP::Lite
	-> uri('http://hwdb.mandriva.com/HW')
	-> proxy('http://hwdb.mandriva.com/HW/hw.cgi')
	-> on_fault(sub { my($soap, $res) = @_;
			  die ref $res ? $res->faultstring : $soap->transport->status, "\n";
		      });

    $pass = $ENV{HWDB_PASSWD};

    if (!$pass) {

	POSIX::isatty(fileno(STDIN)) or die "not a tty\n";
	
	print "Password for $user: ";
	ReadMode('noecho');
	$pass = ReadLine 0;
	chomp $pass;
	ReadMode('normal');
	print "\n";
    }
    
    return $hwdb
}

1;

# hwdb.pm ends here
